<?php

namespace Drupal\custom_entity_pager\TwigExtensions;

use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\custom_entity_pager\Services\CustomEntityPager;
use Drupal\node\NodeInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Custom twig extension used to render the entity pager.
 */
class PaginatorExtension extends AbstractExtension {

  /**
   * Route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $currentRouteMatch;

  /**
   * The custom entity pager.
   *
   * @var \Drupal\custom_entity_pager\Services\CustomEntityPager
   */
  protected $customEntityPager;

  /**
   * Constructs a new PaginatorExtension object.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $currentRouteMatch
   *   Route match.
   * @param \Drupal\custom_entity_pager\Services\CustomEntityPager $customEntityPager
   *   The custom entity pager.
   */
  public function __construct(RouteMatchInterface $currentRouteMatch, CustomEntityPager $customEntityPager) {
    $this->currentRouteMatch = $currentRouteMatch;
    $this->customEntityPager = $customEntityPager;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'custom_entity_pager.paginator_extension';
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('custom_entity_pager_insert', [$this, 'customEntityPagerInsert']),
    ];
  }

  /**
   * Inserts the entity pager.
   *
   * @param string $content_type
   *   Name of the content type to build the pager.
   * @param string $field_order
   *   The field to sort by.
   * @param bool $title
   *   Show titles.
   * @param string $inner_text
   *   Paging text.
   */
  public function customEntityPagerInsert($content_type, $field_order = '', $title = TRUE, $inner_text = NULL) {

    $current_nid = NULL;
    $node = $this->currentRouteMatch->getParameter('node');
    if ($node instanceof NodeInterface) {
      $current_nid = $node->id();
    }

    if (!empty($current_nid)) {

      $elements = $this->customEntityPager->getPaginator($content_type, $current_nid, $field_order);
      $elements['inner_text'] = $inner_text;
      $template = 'paginate';

      if ($title) {
        $template = 'paginate_with_titles';
      }
      return [
        '#theme' => $template,
        '#element' => $elements,
      ];
    }

    return FALSE;
  }

}
